import java.io.FileOutputStream;
import java.io.IOException;

public class mandel {

	/* Must be multiple of 32, to keep BMP generator correct */
	static final int WIDTH0=128;
	/* Real part: -2 .. 1 */
	static final int WIDTH=3*WIDTH0;
	/* Imaginary part: -1 .. 1 */
	static final int HEIGHT=2*WIDTH0;
	static final int ITERATIONS=30;

	static boolean is_in_set(double re, double im, int iterations)
	{
		double zre, zim, zre2, zim2;
		zre = zim = 0.0;
		int i;
		for(i=0; i<iterations; i++) {
			/* z_n1 = z_n**2 + c */
			zre2 = zre*zre-zim*zim+re;
			zim2 = 2*zre*zim+im;
			zre = zre2;
			zim = zim2;
			/* if abs(z) > 2 */
			if (zre*zre + zim*zim > 4)
			return false;
		}
		return true;
	}

	/* http://de.wikipedia.org/wiki/Windows_Bitmap */
	/* black-and-white 1bpp uncompressed 
	*/
	static final int HDRSIZE=(14 + 40 + 8);
	static final int IMGSIZE=(HDRSIZE + WIDTH*HEIGHT/8);
	static byte[] header = new byte[]{
		'B', 'M',					 /* bfType */
		0, 0, 0, 0,				  /* bfSize */
		0, 0, 0, 0,				   /* bfReserved */
		0, 0, 0, 0,				  /* bfOffBits */
		40, 0, 0, 0,				  /* biSize */
		0, 0, 0, 0,					/* biWidth */
		0, 0, 0, 0,				   /* biHeight*/
		1, 0,						 /* biPlanes */
		1, 0,						 /* biBitCount */
		0, 0, 0, 0,				   /* biCompression */
			0, 0, 0, 0,				   /* biSizeImage */
		0, 0, 0, 0,				   /* biXPelsPerMeter */
		0, 0, 0, 0,				   /* biYPelsPerMeter */
		0, 0, 0, 0,				   /* biClrUsed */
		0, 0, 0, 0,				   /* biClrImportant */
		/* color table */
		-1, -1, -1, 0,				/* white */
		0, 0, 0, 0,				   /* black */
	};

	/* Store a DWORD in little-endian mode at offs */
	static void LE(byte bmp[], int offs, int value)
	{
		bmp[offs]   = (byte)(value % 256);
		bmp[offs+1] = (byte)(value / 256 % 256);
		bmp[offs+2] = (byte)(value / 256 / 256 % 256);
		bmp[offs+3] = (byte)(value / 256 / 256 / 256 % 256);
	}

	public static void main(String args[]) throws IOException
	{
		int x, y, pos, bit;

		byte bmp[] = new byte[IMGSIZE];
		System.arraycopy(header, 0, bmp, 0, header.length);
		LE(bmp, 2, IMGSIZE);
		LE(bmp, 10, HDRSIZE);
		LE(bmp, 18, WIDTH);
		LE(bmp, 22, HEIGHT);
		LE(bmp, 34, WIDTH*HEIGHT/8);

		pos = HDRSIZE;
		bit = 0;
		// STUBSTART
		for(y = -WIDTH0; y < WIDTH0; y++) {
			for(x = -2*WIDTH0; x < WIDTH0; x++) {
				bmp[pos] <<= 1;
				if (is_in_set(1.0*x/WIDTH0, 1.0*y/WIDTH0, ITERATIONS))
					bmp[pos] += 1;
				bit++;
				if (bit == 8) {
					pos++;
					bit = 0;
				}
			}
		}
		// STUBSTOP

		FileOutputStream out = new FileOutputStream("mandel.bmp");
		out.write(bmp);
		out.close();
		// STUBEND
	}
}
